package nts.interf;

import java.util.List;

import nts.interf.base.*;

/**
 * Interface for subsystems of numerical transition systems.
 *
 */
public interface ISubsystem extends IScope, IAnnotated, IVisitable {
	/**
	 * @return the name of the subsystem
	 */
	public String name();
	/**
	 * @return number of instances of the subsystem
	 */
	public IExpr instances();
	/**
	 * @return list of all control states 
	 */
	public List<IControlState> controlStates();
	/**
	 * @return list of control states marked as initial 
	 */
	public List<IControlState> marksInit();
	/**
	 * @return list of control states marked as final 
	 */
	public List<IControlState> marksFinal();
	/**
	 * @return list of control states marked as error 
	 */
	public List<IControlState> marksError();
	/**
	 * @return list of variables declared as input variables
	 */
	public List<IVarTableEntry> varIn();
	/**
	 * @return list of variables declared as output variables
	 */
	public List<IVarTableEntry> varOut();
	/**
	 * @return list of transitions
	 */
	public List<ITransition> transitions();
}
