package nts.parser;

import org.antlr.runtime.Token;
import java.util.*;

import nts.interf.base.IExpr;
import nts.interf.expr.IAccessMulti;
import nts.interf.base.IVisitor;

// syntax: a[i1]..[im][j1,..,jn]
public class AccessMulti extends Access implements IAccessMulti {
	
	public void accept(IVisitor v) { v.visit(this); }
	
	// guarantees: size >= 0
	private List<IExpr> single = new LinkedList<IExpr>();
	// guarantees: size >= 2
	private List<IExpr> multi = new LinkedList<IExpr>();
	
	public List<IExpr> singleInxs() { return single; }
	public List<IExpr> multiInxs() { return multi; }
	
	public int accessDepth() { return single.size() + 1; }
	public int multiAccess() { return multi.size(); }
	
	AccessMulti(Token aId, List<IExpr> aM, List<IExpr> aS) {
		super(aId);
		single = aS;
		multi = aM;
	}
	AccessMulti(Token aId) {
		super(aId);
	}
	
	void setSingle(List<IExpr> aI) {
		single = aI;
	}
	void setMulti(List<IExpr> aI) {
		multi = aI;
	}
	
	Expr semanticChecks(VarTable aVT, SemFlags aS) {
		
		this.check_base(aVT, aS);
		
		// semantic checks for indices
		SemFlags s = SemFlags.addNoPrimed(aS);
		Expr.checkInt(single, aVT, s);
		Expr.checkInt(multi, aVT, s);
		
		return this;
	}
	
	public AccessMulti syn_copy() {
		AccessMulti e = new AccessMulti(token);
		for (IExpr ee : single) {
			e.single.add(((Expr)ee).syn_copy());
		}
		for (IExpr ee : multi) {
			e.multi.add(((Expr)ee).syn_copy());
		}
		return e;
	}
}
