package nts.parser;

import nts.interf.base.EBasicType;
import nts.interf.base.IVisitor;
import nts.interf.expr.IExprEq;

import org.antlr.runtime.Token;

public class ExEq extends OpBinRel implements IExprEq {

	public void accept(IVisitor v) { v.visit(this); }
	
	public ExEq(Token aT, Expr aE1, Expr aE2) { super(aT,aE1,aE2); }
	
	public static ExEq createRev(Token aT, Expr aE2, Expr aE1) { return new ExEq(aT,aE1,aE2); }
	
	ExEq syn_copy() { return new ExEq(token,op1.syn_copy(),op2.syn_copy()); }
	
	private boolean isArrayWriteOrBool(VarTable aVT, Expr aE) {
		if (op1 instanceof Access) {
			Access a = (Access)op1;
			VarTableEntry e = aVT.checkDeclared(a.token());
			//VarTableEntry e = aVT.get(a.varName());
			if (e.isPrimed() && (e.type().isArray() || e.type().isBool()) ) {
				return true;
			}
		}
		return false;
	}
	
	private Expr semaChecksArrayWriteOrBool(VarTable aVT, SemFlags aS, Expr eTo, Expr eFrom) {
		eTo.blockWriteTo = false;
		eTo.semanticChecks(aVT, aS);
		
		//eFrom.blockWriteFrom = false;
		SemFlags f = SemFlags.addNoPrimed(aS);
		eFrom.semanticChecks(aVT, f);
		
		// check if types match
		Type t1 = op1.type();
		Type t2 = op2.type();
		
		if (!t1.matches(t2)) {
			errNoMatch(token);
		}
		
		type = new Type(EBasicType.BOOL);
		
		return this;
	}
	
	Expr semanticChecks(VarTable aVT, SemFlags aS) {
		
		// check for the array-assignment pattern
		if (!aS.noPrimed()) {
			
			if (isArrayWriteOrBool(aVT, op1)) {
				
				return semaChecksArrayWriteOrBool(aVT,aS,op1,op2);
				
			} else if (isArrayWriteOrBool(aVT, op2)) {
				
				return semaChecksArrayWriteOrBool(aVT,aS,op2,op1);
			}
		}
		
		return super.semanticChecks(aVT, aS);
	}
}
